@@ -1,7 +1,28 @@
+/* Copyright 2009 Ryan Dahl <ry@tinyclouds.org>
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE. 
+ */
 #include "http_parser.h"
 #include <stdlib.h>
 #include <assert.h>
 #include <stdio.h>
+#include <stdlib.h> /* rand */
 #include <string.h>
 #include <stdarg.h>
 
@@ -13,15 +34,18 @@
 #define MAX_HEADERS 10
 #define MAX_ELEMENT_SIZE 500
 
-static http_parser parser;
+enum message_type { REQUEST, RESPONSE };
+
+static http_parser *parser;
+
 struct message {
   const char *name; // for debugging purposes
   const char *raw;
-  enum http_parser_type type;
-  int method;
+  enum message_type type;
+  enum http_method method;
   int status_code;
   char request_path[MAX_ELEMENT_SIZE];
-  char request_uri[MAX_ELEMENT_SIZE];
+  char request_url[MAX_ELEMENT_SIZE];
   char fragment[MAX_ELEMENT_SIZE];
   char query_string[MAX_ELEMENT_SIZE];
   char body[MAX_ELEMENT_SIZE];
@@ -30,11 +54,26 @@ struct message {
   char headers [MAX_HEADERS][2][MAX_ELEMENT_SIZE];
   int should_keep_alive;
 
+  unsigned short http_major;
+  unsigned short http_minor;
+
   int message_begin_cb_called;
   int headers_complete_cb_called;
   int message_complete_cb_called;
+  int message_complete_on_eof;
 };
 
+static int currently_parsing_eof;
+
+inline size_t parse (enum message_type t, const char *buf, size_t len)
+{
+  size_t nparsed;
+  currently_parsing_eof = (len == 0);
+  nparsed = (t == REQUEST ? http_parse_requests(parser, buf, len) 
+                          : http_parse_responses(parser, buf, len));
+  return nparsed;
+}
+
 static struct message messages[5];
 static int num_messages;
 
@@ -42,18 +81,21 @@ static int num_messages;
 const struct message requests[] =
 #define CURL_GET 0
 { {.name= "curl get"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /test HTTP/1.1\r\n"
          "User-Agent: curl/7.18.0 (i486-pc-linux-gnu) libcurl/7.18.0 OpenSSL/0.9.8g zlib/1.2.3.3 libidn/1.1\r\n"
          "Host: 0.0.0.0=5000\r\n"
          "Accept: */*\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE 
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/test"
-  ,.request_uri= "/test"
+  ,.request_url= "/test"
   ,.num_headers= 3
   ,.headers=
     { { "User-Agent", "curl/7.18.0 (i486-pc-linux-gnu) libcurl/7.18.0 OpenSSL/0.9.8g zlib/1.2.3.3 libidn/1.1" }
@@ -65,7 +107,7 @@ const struct message requests[] =
 
 #define FIREFOX_GET 1
 , {.name= "firefox get"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /favicon.ico HTTP/1.1\r\n"
          "Host: 0.0.0.0=5000\r\n"
          "User-Agent: Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/2008061015 Firefox/3.0\r\n"
@@ -77,11 +119,14 @@ const struct message requests[] =
          "Connection: keep-alive\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE 
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/favicon.ico"
-  ,.request_uri= "/favicon.ico"
+  ,.request_url= "/favicon.ico"
   ,.num_headers= 8
   ,.headers=
     { { "Host", "0.0.0.0=5000" }
@@ -98,16 +143,19 @@ const struct message requests[] =
 
 #define DUMBFUCK 2
 , {.name= "dumbfuck"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /dumbfuck HTTP/1.1\r\n"
          "aaaaaaaaaaaaa:++++++++++\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE 
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/dumbfuck"
-  ,.request_uri= "/dumbfuck"
+  ,.request_url= "/dumbfuck"
   ,.num_headers= 1
   ,.headers=
     { { "aaaaaaaaaaaaa",  "++++++++++" }
@@ -116,48 +164,57 @@ const struct message requests[] =
   }
 
 #define FRAGMENT_IN_URI 3
-, {.name= "fragment in uri"
-  ,.type= HTTP_REQUEST
+, {.name= "fragment in url"
+  ,.type= REQUEST
   ,.raw= "GET /forums/1/topics/2375?page=1#posts-17408 HTTP/1.1\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE 
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= "page=1"
   ,.fragment= "posts-17408"
   ,.request_path= "/forums/1/topics/2375"
-  /* XXX request uri does not include fragment? */
-  ,.request_uri= "/forums/1/topics/2375?page=1"
+  /* XXX request url does include fragment? */
+  ,.request_url= "/forums/1/topics/2375?page=1#posts-17408"
   ,.num_headers= 0
   ,.body= ""
   }
 
 #define GET_NO_HEADERS_NO_BODY 4
 , {.name= "get no headers no body"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /get_no_headers_no_body/world HTTP/1.1\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE /* would need Connection: close */
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/get_no_headers_no_body/world"
-  ,.request_uri= "/get_no_headers_no_body/world"
+  ,.request_url= "/get_no_headers_no_body/world"
   ,.num_headers= 0
   ,.body= ""
   }
 
 #define GET_ONE_HEADER_NO_BODY 5
 , {.name= "get one header no body"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /get_one_header_no_body HTTP/1.1\r\n"
          "Accept: */*\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE /* would need Connection: close */
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/get_one_header_no_body"
-  ,.request_uri= "/get_one_header_no_body"
+  ,.request_url= "/get_one_header_no_body"
   ,.num_headers= 1
   ,.headers=
     { { "Accept" , "*/*" }
@@ -167,17 +224,20 @@ const struct message requests[] =
 
 #define GET_FUNKY_CONTENT_LENGTH 6
 , {.name= "get funky content length body hello"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /get_funky_content_length_body_hello HTTP/1.0\r\n"
          "conTENT-Length: 5\r\n"
          "\r\n"
          "HELLO"
   ,.should_keep_alive= FALSE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 0
   ,.method= HTTP_GET
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/get_funky_content_length_body_hello"
-  ,.request_uri= "/get_funky_content_length_body_hello"
+  ,.request_url= "/get_funky_content_length_body_hello"
   ,.num_headers= 1
   ,.headers=
     { { "conTENT-Length" , "5" }
@@ -187,7 +247,7 @@ const struct message requests[] =
 
 #define POST_IDENTITY_BODY_WORLD 7
 , {.name= "post identity body world"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "POST /post_identity_body_world?q=search#hey HTTP/1.1\r\n"
          "Accept: */*\r\n"
          "Transfer-Encoding: identity\r\n"
@@ -195,11 +255,14 @@ const struct message requests[] =
          "\r\n"
          "World"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_POST
   ,.query_string= "q=search"
   ,.fragment= "hey"
   ,.request_path= "/post_identity_body_world"
-  ,.request_uri= "/post_identity_body_world?q=search"
+  ,.request_url= "/post_identity_body_world?q=search#hey"
   ,.num_headers= 3
   ,.headers=
     { { "Accept", "*/*" }
@@ -211,7 +274,7 @@ const struct message requests[] =
 
 #define POST_CHUNKED_ALL_YOUR_BASE 8
 , {.name= "post - chunked body: all your base are belong to us"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "POST /post_chunked_all_your_base HTTP/1.1\r\n"
          "Transfer-Encoding: chunked\r\n"
          "\r\n"
@@ -219,11 +282,14 @@ const struct message requests[] =
          "0\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_POST
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/post_chunked_all_your_base"
-  ,.request_uri= "/post_chunked_all_your_base"
+  ,.request_url= "/post_chunked_all_your_base"
   ,.num_headers= 1
   ,.headers=
     { { "Transfer-Encoding" , "chunked" }
@@ -233,7 +299,7 @@ const struct message requests[] =
 
 #define TWO_CHUNKS_MULT_ZERO_END 9
 , {.name= "two chunks ; triple zero ending"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "POST /two_chunks_mult_zero_end HTTP/1.1\r\n"
          "Transfer-Encoding: chunked\r\n"
          "\r\n"
@@ -242,11 +308,14 @@ const struct message requests[] =
          "000\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_POST
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/two_chunks_mult_zero_end"
-  ,.request_uri= "/two_chunks_mult_zero_end"
+  ,.request_url= "/two_chunks_mult_zero_end"
   ,.num_headers= 1
   ,.headers=
     { { "Transfer-Encoding", "chunked" }
@@ -256,7 +325,7 @@ const struct message requests[] =
 
 #define CHUNKED_W_TRAILING_HEADERS 10
 , {.name= "chunked with trailing headers. blech."
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "POST /chunked_w_trailing_headers HTTP/1.1\r\n"
          "Transfer-Encoding: chunked\r\n"
          "\r\n"
@@ -267,21 +336,26 @@ const struct message requests[] =
          "Content-Type: text/plain\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_POST
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/chunked_w_trailing_headers"
-  ,.request_uri= "/chunked_w_trailing_headers"
-  ,.num_headers= 1
+  ,.request_url= "/chunked_w_trailing_headers"
+  ,.num_headers= 3
   ,.headers=
     { { "Transfer-Encoding",  "chunked" }
+    , { "Vary", "*" }
+    , { "Content-Type", "text/plain" }
     }
   ,.body= "hello world"
   }
 
 #define CHUNKED_W_BULLSHIT_AFTER_LENGTH 11
 , {.name= "with bullshit after the length"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "POST /chunked_w_bullshit_after_length HTTP/1.1\r\n"
          "Transfer-Encoding: chunked\r\n"
          "\r\n"
@@ -290,11 +364,14 @@ const struct message requests[] =
          "0\r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_POST
   ,.query_string= ""
   ,.fragment= ""
   ,.request_path= "/chunked_w_bullshit_after_length"
-  ,.request_uri= "/chunked_w_bullshit_after_length"
+  ,.request_url= "/chunked_w_bullshit_after_length"
   ,.num_headers= 1
   ,.headers=
     { { "Transfer-Encoding", "chunked" }
@@ -304,26 +381,59 @@ const struct message requests[] =
 
 #define WITH_QUOTES 12
 , {.name= "with quotes"
-  ,.type= HTTP_REQUEST
+  ,.type= REQUEST
   ,.raw= "GET /with_\"stupid\"_quotes?foo=\"bar\" HTTP/1.1\r\n\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.method= HTTP_GET
   ,.query_string= "foo=\"bar\""
   ,.fragment= ""
   ,.request_path= "/with_\"stupid\"_quotes"
-  ,.request_uri= "/with_\"stupid\"_quotes?foo=\"bar\""
+  ,.request_url= "/with_\"stupid\"_quotes?foo=\"bar\""
   ,.num_headers= 0
   ,.headers= { }
   ,.body= ""
   }
 
+#define APACHEBENCH_GET 13
+/* The server receiving this request SHOULD NOT wait for EOF
+ * to know that content-length == 0.
+ * How to represent this in a unit test? message_complete_on_eof
+ * Compare with NO_CONTENT_LENGTH_RESPONSE.
+ */
+, {.name = "apachebench get"
+  ,.type= REQUEST
+  ,.raw= "GET /test HTTP/1.0\r\n"
+         "Host: 0.0.0.0:5000\r\n"
+         "User-Agent: ApacheBench/2.3\r\n"
+         "Accept: */*\r\n\r\n"
+  ,.should_keep_alive= FALSE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 0
+  ,.method= HTTP_GET
+  ,.query_string= ""
+  ,.fragment= ""
+  ,.request_path= "/test"
+  ,.request_url= "/test"
+  ,.num_headers= 3
+  ,.headers= { { "Host", "0.0.0.0:5000" }
+             , { "User-Agent", "ApacheBench/2.3" }
+             , { "Accept", "*/*" }
+             }
+  ,.body= ""
+  }
+
 , {.name= NULL } /* sentinel */
 };
 
 /* * R E S P O N S E S * */
 const struct message responses[] =
+#define GOOGLE_301 0
 { {.name= "google 301"
-  ,.type= HTTP_RESPONSE
+  ,.type= RESPONSE
   ,.raw= "HTTP/1.1 301 Moved Permanently\r\n"
          "Location: http://www.google.com/\r\n"
          "Content-Type: text/html; charset=UTF-8\r\n"
@@ -340,6 +450,9 @@ const struct message responses[] =
          "<A HREF=\"http://www.google.com/\">here</A>.\r\n"
          "</BODY></HTML>\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.status_code= 301
   ,.num_headers= 7
   ,.headers=
@@ -359,8 +472,14 @@ const struct message responses[] =
           "</BODY></HTML>\r\n"
   }
 
+#define NO_CONTENT_LENGTH_RESPONSE 1
+/* The client should wait for the server's EOF. That is, when content-length
+ * is not specified, and "Connection: close", the end of body is specified
+ * by the EOF.
+ * Compare with APACHEBENCH_GET
+ */
 , {.name= "no content-length response"
-  ,.type= HTTP_RESPONSE
+  ,.type= RESPONSE
   ,.raw= "HTTP/1.1 200 OK\r\n"
          "Date: Tue, 04 Aug 2009 07:59:32 GMT\r\n"
          "Server: Apache\r\n"
@@ -378,6 +497,9 @@ const struct message responses[] =
          "  </SOAP-ENV:Body>\n"
          "</SOAP-ENV:Envelope>"
   ,.should_keep_alive= FALSE
+  ,.message_complete_on_eof= TRUE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.status_code= 200
   ,.num_headers= 5
   ,.headers=
@@ -398,28 +520,37 @@ const struct message responses[] =
           "</SOAP-ENV:Envelope>"
   }
 
+#define NO_HEADERS_NO_BODY_404 2
 , {.name= "404 no headers no body"
-  ,.type= HTTP_RESPONSE
+  ,.type= RESPONSE
   ,.raw= "HTTP/1.1 404 Not Found\r\n\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.status_code= 404
   ,.num_headers= 0
   ,.headers= {}
   ,.body= ""
   }
 
+#define NO_REASON_PHRASE 3
 , {.name= "301 no response phrase"
-  ,.type= HTTP_RESPONSE
+  ,.type= RESPONSE
   ,.raw= "HTTP/1.1 301\r\n\r\n"
   ,.should_keep_alive = TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.status_code= 301
   ,.num_headers= 0
   ,.headers= {}
   ,.body= ""
   }
 
+#define TRAILING_SPACE_ON_CHUNKED_BODY 4
 , {.name="200 trailing space on chunked body"
-  ,.type= HTTP_RESPONSE
+  ,.type= RESPONSE
   ,.raw= "HTTP/1.1 200 OK\r\n"
          "Content-Type: text/plain\r\n"
          "Transfer-Encoding: chunked\r\n"
@@ -433,6 +564,9 @@ const struct message responses[] =
          "0  \r\n"
          "\r\n"
   ,.should_keep_alive= TRUE
+  ,.message_complete_on_eof= FALSE
+  ,.http_major= 1
+  ,.http_minor= 1
   ,.status_code= 200
   ,.num_headers= 2
   ,.headers=
@@ -449,47 +583,47 @@ const struct message responses[] =
 };
 
 int
-request_path_cb (http_parser *parser, const char *p, size_t len)
+request_path_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
-  strncat(messages[num_messages].request_path, p, len);
+  assert(p == parser);
+  strncat(messages[num_messages].request_path, buf, len);
   return 0;
 }
 
 int
-request_uri_cb (http_parser *parser, const char *p, size_t len)
+request_url_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
-  strncat(messages[num_messages].request_uri, p, len);
+  assert(p == parser);
+  strncat(messages[num_messages].request_url, buf, len);
   return 0;
 }
 
 int
-query_string_cb (http_parser *parser, const char *p, size_t len)
+query_string_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
-  strncat(messages[num_messages].query_string, p, len);
+  assert(p == parser);
+  strncat(messages[num_messages].query_string, buf, len);
   return 0;
 }
 
 int
-fragment_cb (http_parser *parser, const char *p, size_t len)
+fragment_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
-  strncat(messages[num_messages].fragment, p, len);
+  assert(p == parser);
+  strncat(messages[num_messages].fragment, buf, len);
   return 0;
 }
 
 int
-header_field_cb (http_parser *parser, const char *p, size_t len)
+header_field_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
+  assert(p == parser);
   struct message *m = &messages[num_messages];
 
   if (m->last_header_element != FIELD)
     m->num_headers++;
 
-  strncat(m->headers[m->num_headers-1][0], p, len);
+  strncat(m->headers[m->num_headers-1][0], buf, len);
 
   m->last_header_element = FIELD;
 
@@ -497,12 +631,12 @@ header_field_cb (http_parser *parser, const char *p, size_t len)
 }
 
 int
-header_value_cb (http_parser *parser, const char *p, size_t len)
+header_value_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
+  assert(p == parser);
   struct message *m = &messages[num_messages];
 
-  strncat(m->headers[m->num_headers-1][1], p, len);
+  strncat(m->headers[m->num_headers-1][1], buf, len);
 
   m->last_header_element = VALUE;
 
@@ -510,146 +644,254 @@ header_value_cb (http_parser *parser, const char *p, size_t len)
 }
 
 int
-body_cb (http_parser *parser, const char *p, size_t len)
+body_cb (http_parser *p, const char *buf, size_t len)
 {
-  assert(parser);
-  strncat(messages[num_messages].body, p, len);
+  assert(p == parser);
+  strncat(messages[num_messages].body, buf, len);
  // printf("body_cb: '%s'\n", requests[num_messages].body);
   return 0;
 }
 
 int
-message_complete_cb (http_parser *parser)
+message_begin_cb (http_parser *p)
 {
-  messages[num_messages].method = parser->method;
-  messages[num_messages].status_code = parser->status_code;
-
-  messages[num_messages].message_complete_cb_called = TRUE;
-
-  num_messages++;
-  return 0;
-}
-
-int
-message_begin_cb (http_parser *parser)
-{
-  assert(parser);
+  assert(p == parser);
   messages[num_messages].message_begin_cb_called = TRUE;
   return 0;
 }
 
 int
-headers_complete_cb (http_parser *parser)
+headers_complete_cb (http_parser *p)
 {
-  assert(parser);
+  assert(p == parser);
+  messages[num_messages].method = parser->method;
+  messages[num_messages].status_code = parser->status_code;
+  messages[num_messages].http_major = parser->http_major;
+  messages[num_messages].http_minor = parser->http_minor;
   messages[num_messages].headers_complete_cb_called = TRUE;
+  messages[num_messages].should_keep_alive = http_should_keep_alive(parser);
+  return 0;
+}
+
+int
+message_complete_cb (http_parser *p)
+{
+  assert(p == parser);
+  if (messages[num_messages].should_keep_alive != http_should_keep_alive(parser))
+  {
+    fprintf(stderr, "\n\n *** Error http_should_keep_alive() should have same "
+                    "value in both on_message_complete and on_headers_complete "
+                    "but it doesn't! ***\n\n");
+    assert(0);
+    exit(1);
+  }
+  messages[num_messages].message_complete_cb_called = TRUE;
+
+  messages[num_messages].message_complete_on_eof = currently_parsing_eof;
+  
+  num_messages++;
   return 0;
 }
 
 void
-parser_init (enum http_parser_type type)
+parser_init ()
 {
   num_messages = 0;
 
-  http_parser_init(&parser, type);
+  assert(parser == NULL);
+
+  parser = malloc(sizeof(http_parser));
+
+  http_parser_init(parser);
 
   memset(&messages, 0, sizeof messages);
 
-  parser.on_message_begin     = message_begin_cb;
-  parser.on_header_field      = header_field_cb;
-  parser.on_header_value      = header_value_cb;
-  parser.on_path              = request_path_cb;
-  parser.on_uri               = request_uri_cb;
-  parser.on_fragment          = fragment_cb;
-  parser.on_query_string      = query_string_cb;
-  parser.on_body              = body_cb;
-  parser.on_headers_complete  = headers_complete_cb;
-  parser.on_message_complete  = message_complete_cb;
+  parser->on_message_begin     = message_begin_cb;
+  parser->on_header_field      = header_field_cb;
+  parser->on_header_value      = header_value_cb;
+  parser->on_path              = request_path_cb;
+  parser->on_url               = request_url_cb;
+  parser->on_fragment          = fragment_cb;
+  parser->on_query_string      = query_string_cb;
+  parser->on_body              = body_cb;
+  parser->on_headers_complete  = headers_complete_cb;
+  parser->on_message_complete  = message_complete_cb;
 }
 
 void
+parser_free ()
+{
+  assert(parser);
+  free(parser);
+  parser = NULL;
+}
+
+static inline int
+check_str_eq (const struct message *m,
+              const char *prop,
+              const char *expected,
+              const char *found) {
+  if (0 != strcmp(expected, found)) {
+    printf("\n*** Error: %s in '%s' ***\n\n", prop, m->name);
+    printf("expected '%s'\n", expected);
+    printf("   found '%s'\n", found);
+    return 0;
+  }
+  return 1;
+}
+
+static inline int
+check_num_eq (const struct message *m,
+              const char *prop,
+              int expected,
+              int found) {
+  if (expected != found) {
+    printf("\n*** Error: %s in '%s' ***\n\n", prop, m->name);
+    printf("expected %d\n", expected);
+    printf("   found %d\n", found);
+    return 0;
+  }
+  return 1;
+}
+
+#define MESSAGE_CHECK_STR_EQ(expected, found, prop) \
+  if (!check_str_eq(expected, #prop, expected->prop, found->prop)) return 0
+
+#define MESSAGE_CHECK_NUM_EQ(expected, found, prop) \
+  if (!check_num_eq(expected, #prop, expected->prop, found->prop)) return 0
+
+
+int
 message_eq (int index, const struct message *expected)
 {
   int i;
   struct message *m = &messages[index];
 
-  assert(m->method == expected->method);
-  assert(m->status_code == expected->status_code);
+  MESSAGE_CHECK_NUM_EQ(expected, m, http_major);
+  MESSAGE_CHECK_NUM_EQ(expected, m, http_minor);
+
+  if (expected->type == REQUEST) {
+    MESSAGE_CHECK_NUM_EQ(expected, m, method);
+  } else {
+    MESSAGE_CHECK_NUM_EQ(expected, m, status_code);
+  }
+
+  MESSAGE_CHECK_NUM_EQ(expected, m, should_keep_alive);
+  MESSAGE_CHECK_NUM_EQ(expected, m, message_complete_on_eof);
 
   assert(m->message_begin_cb_called);
   assert(m->headers_complete_cb_called);
   assert(m->message_complete_cb_called);
 
-  assert(0 == strcmp(m->body, expected->body));
-  assert(0 == strcmp(m->fragment, expected->fragment));
-  assert(0 == strcmp(m->query_string, expected->query_string));
-  assert(0 == strcmp(m->request_path, expected->request_path));
-  assert(0 == strcmp(m->request_uri, expected->request_uri));
-  assert(m->num_headers == expected->num_headers);
+
+  MESSAGE_CHECK_STR_EQ(expected, m, request_path);
+  MESSAGE_CHECK_STR_EQ(expected, m, query_string);
+  MESSAGE_CHECK_STR_EQ(expected, m, fragment);
+  MESSAGE_CHECK_STR_EQ(expected, m, request_url);
+  MESSAGE_CHECK_STR_EQ(expected, m, body);
+
+  MESSAGE_CHECK_NUM_EQ(expected, m, num_headers);
+
+  int r;
   for (i = 0; i < m->num_headers; i++) {
-    assert(0 == strcmp(m->headers[i][0], expected->headers[i][0]));
-    assert(0 == strcmp(m->headers[i][1], expected->headers[i][1]));
+    r = check_str_eq(expected, "header field", expected->headers[i][0], m->headers[i][0]);
+    if (!r) return 0;
+    r = check_str_eq(expected, "header value", expected->headers[i][1], m->headers[i][1]);
+    if (!r) return 0;
   }
+
+  return 1;
 }
 
-void
-parse_messages (int message_count, const struct message *input_messages[])
+static void
+print_error (const char *raw, size_t error_location)
 {
-  // Concat the input messages
-  size_t length = 0;
-  int i;
-  for (i = 0; i < message_count; i++) {
-    length += strlen(input_messages[i]->raw);
-  }
-  char total[length + 1];
-  total[0] = '\0';
+  fprintf(stderr, "\n*** parse error ***\n\n");
+  
+  int this_line = 0, char_len = 0;
+  size_t i, j, len = strlen(raw), error_location_line = 0;
+  for (i = 0; i < len; i++) {
+    if (i == error_location) this_line = 1;
+    switch (raw[i]) {
+      case '\r':
+        char_len = 2;
+        fprintf(stderr, "\\r");
+        break;
 
-  for (i = 0; i < message_count; i++) {
-    strcat(total, input_messages[i]->raw);
+      case '\n':
+        char_len = 2;
+        fprintf(stderr, "\\n\n");
+
+        if (this_line) goto print;
+
+        error_location_line = 0;
+        continue;
+
+      default:
+        char_len = 1;
+        fputc(raw[i], stderr);
+        break;
+    }
+    if (!this_line) error_location_line += char_len;
   }
 
-  // Parse the stream
-  parser_init(HTTP_REQUEST);
+  fprintf(stderr, "[eof]\n");
 
-  http_parser_execute(&parser, total, length);
-  assert(!http_parser_has_error(&parser));
-
-  http_parser_execute(&parser, NULL, 0);
-  assert(!http_parser_has_error(&parser));
-
-  assert(num_messages == message_count);
-
-  for (i = 0; i < message_count; i++) {
-    message_eq(i, input_messages[i]);
+ print:
+  for (j = 0; j < error_location_line; j++) {
+    fputc(' ', stderr);
   }
+  fprintf(stderr, "^\n\nerror location: %d\n", error_location);
 }
 
 
 void
 test_message (const struct message *message)
 {
-  parser_init(message->type);
+  parser_init();
 
-  http_parser_execute(&parser, message->raw, strlen(message->raw));
-  assert(!http_parser_has_error(&parser));
+  size_t read;
 
-  http_parser_execute(&parser, NULL, 0);
-  assert(!http_parser_has_error(&parser));
+  read = parse(message->type, message->raw, strlen(message->raw));
+  if (read != strlen(message->raw)) {
+    print_error(message->raw, read);
+    exit(1);
+  }
 
-  assert(num_messages == 1);
+  read = parse(message->type, NULL, 0);
+  if (read != 0) {
+    print_error(message->raw, read);
+    exit(1);
+  }
 
-  message_eq(0, message);
+  if (num_messages != 1) {
+    printf("\n*** num_messages != 1 after testing '%s' ***\n\n", message->name);
+    exit(1);
+  }
+
+  if(!message_eq(0, message)) exit(1);
+
+  parser_free();
 }
 
 void
 test_error (const char *buf)
 {
-  parser_init(HTTP_REQUEST);
+  parser_init();
 
-  http_parser_execute(&parser, buf, strlen(buf));
-  http_parser_execute(&parser, NULL, 0);
+  size_t parsed;
 
-  assert(http_parser_has_error(&parser));
+  parsed = parse(REQUEST, buf, strlen(buf));
+  if (parsed != strlen(buf)) goto out;
+  parsed = parse(REQUEST, NULL, 0);
+  if (parsed != 0) goto out;
+
+  fprintf(stderr, "\n*** Error expected but none found ***\n\n%s", buf);
+  exit(1);
+  return;
+
+out:
+  parser_free();
 }
 
 void
@@ -666,18 +908,32 @@ test_multiple3 (const struct message *r1, const struct message *r2, const struct
   strcat(total, r2->raw);
   strcat(total, r3->raw);
 
-  parser_init(HTTP_REQUEST);
+  parser_init();
 
-  http_parser_execute(&parser, total, strlen(total));
-  assert(!http_parser_has_error(&parser) );
+  size_t read;
 
-  http_parser_execute(&parser, NULL, 0);
-  assert(!http_parser_has_error(&parser) );
+  read = parse(r1->type, total, strlen(total));
+  if (read != strlen(total)) {
+    print_error(total, read);
+    exit(1);
+  }
 
-  assert(num_messages == 3);
-  message_eq(0, r1);
-  message_eq(1, r2);
-  message_eq(2, r3);
+  read = parse(REQUEST, NULL, 0);
+  if (read != 0) {
+    print_error(total, read);
+    exit(1);
+  }
+
+  if (3 != num_messages) {
+    fprintf(stderr, "\n\n*** Parser didn't see 3 messages only %d *** \n", num_messages);
+    exit(1);
+  }
+
+  if (!message_eq(0, r1)) exit(1);
+  if (!message_eq(1, r2)) exit(1);
+  if (!message_eq(2, r3)) exit(1);
+
+  parser_free();
 }
 
 /* SCAN through every possible breaking to make sure the
@@ -696,11 +952,15 @@ test_scan (const struct message *r1, const struct message *r2, const struct mess
   strcat(total, r2->raw);
   strcat(total, r3->raw);
 
+  size_t read;
+
   int total_len = strlen(total);
 
   int total_ops = (total_len - 1) * (total_len - 2) / 2;
   int ops = 0 ;
 
+  size_t buf1_len, buf2_len, buf3_len;
+
   int i,j;
   for (j = 2; j < total_len; j ++ ) {
     for (i = 1; i < j; i ++ ) {
@@ -711,72 +971,112 @@ test_scan (const struct message *r1, const struct message *r2, const struct mess
       }
       ops += 1;
 
-      parser_init(HTTP_REQUEST);
+      parser_init();
 
-      int buf1_len = i;
+      buf1_len = i;
       strncpy(buf1, total, buf1_len);
       buf1[buf1_len] = 0;
 
-      int buf2_len = j - i;
+      buf2_len = j - i;
       strncpy(buf2, total+i, buf2_len);
       buf2[buf2_len] = 0;
 
-      int buf3_len = total_len - j;
+      buf3_len = total_len - j;
       strncpy(buf3, total+j, buf3_len);
       buf3[buf3_len] = 0;
 
-      /*
-      printf("buf1: %s - %d\n", buf1, buf1_len);
-      printf("buf2: %s - %d \n", buf2, buf2_len );
-      printf("buf3: %s - %d\n\n", buf3, buf3_len);
-      */
+      read = parse(r1->type, buf1, buf1_len);
+      if (read != buf1_len) {
+        print_error(buf1, read);
+        goto error;
+      }
 
-      http_parser_execute(&parser, buf1, buf1_len);
-      assert(!http_parser_has_error(&parser));
+      read = parse(r1->type, buf2, buf2_len);
+      if (read != buf2_len) {
+        print_error(buf2, read);
+        goto error;
+      }
 
-      http_parser_execute(&parser, buf2, buf2_len);
-      assert(!http_parser_has_error(&parser));
+      read = parse(r1->type, buf3, buf3_len);
+      if (read != buf3_len) {
+        print_error(buf3, read);
+        goto error;
+      }
 
-      http_parser_execute(&parser, buf3, buf3_len);
-      assert(!http_parser_has_error(&parser));
+      parse(r1->type, NULL, 0);
 
-      http_parser_execute(&parser, NULL, 0);
-      assert(!http_parser_has_error(&parser));
+      if (3 != num_messages) {
+        fprintf(stderr, "\n\nParser didn't see 3 messages only %d\n", num_messages);
+        goto error;
+      }
 
-      assert(3 == num_messages);
+      if (!message_eq(0, r1)) {
+        fprintf(stderr, "\n\nError matching messages[0] in test_scan.\n");
+        goto error;
+      }
 
-      message_eq(0, r1);
-      message_eq(1, r2);
-      message_eq(2, r3);
+      if (!message_eq(1, r2)) {
+        fprintf(stderr, "\n\nError matching messages[1] in test_scan.\n");
+        goto error;
+      }
+
+      if (!message_eq(2, r3)) {
+        fprintf(stderr, "\n\nError matching messages[2] in test_scan.\n");
+        goto error;
+      }
+
+      parser_free();
     }
   }
   puts("\b\b\b\b100%");
+  return;
+
+error:
+  fprintf(stderr, "i=%d  j=%d\n", i, j);
+  fprintf(stderr, "buf1 (%d) %s\n\n", buf1_len, buf1);
+  fprintf(stderr, "buf2 (%d) %s\n\n", buf2_len , buf2);
+  fprintf(stderr, "buf3 (%d) %s\n", buf3_len, buf3);
+  exit(1);
 }
 
 int
 main (void)
 {
+  parser = NULL;
   int i, j, k;
+  int request_count;
+  int response_count;
 
   printf("sizeof(http_parser) = %d\n", sizeof(http_parser));
 
-  int request_count;
   for (request_count = 0; requests[request_count].name; request_count++);
-
-  int response_count;
   for (response_count = 0; responses[response_count].name; response_count++);
 
-
+#if 0
   //// RESPONSES
 
   for (i = 0; i < response_count; i++) {
     test_message(&responses[i]);
   }
 
+  for (i = 0; i < response_count; i++) {
+    if (!responses[i].should_keep_alive) continue;
+    for (j = 0; j < response_count; j++) {
+      if (!responses[j].should_keep_alive) continue;
+      for (k = 0; k < response_count; k++) {
+        test_multiple3(&responses[i], &responses[j], &responses[k]);
+      }
+    }
+  }
 
+  printf("response scan 1/1      ");
+  test_scan( &responses[TRAILING_SPACE_ON_CHUNKED_BODY]
+           , &responses[NO_HEADERS_NO_BODY_404]
+           , &responses[NO_REASON_PHRASE]
+           );
 
   puts("responses okay");
-
+#endif
 
 
   /// REQUESTS
@@ -822,6 +1122,10 @@ main (void)
     "\r\n";
   test_error(dumbfuck2);
 
+#if 0
+  // NOTE(Wed Nov 18 11:57:27 CET 2009) this seems okay. we just read body
+  // until EOF.
+  //
   // no content-length
   // error if there is a body without content length
   const char *bad_get_no_headers_no_body = "GET /bad_get_no_headers_no_body/world HTTP/1.1\r\n"
@@ -829,8 +1133,7 @@ main (void)
                                            "\r\n"
                                            "HELLO";
   test_error(bad_get_no_headers_no_body);
-
-
+#endif
   /* TODO sending junk and large headers gets rejected */
 
 
@@ -839,10 +1142,13 @@ main (void)
     test_message(&requests[i]);
   }
 
+
+
   for (i = 0; i < request_count; i++) {
+    if (!requests[i].should_keep_alive) continue;
     for (j = 0; j < request_count; j++) {
+      if (!requests[j].should_keep_alive) continue;
       for (k = 0; k < request_count; k++) {
-        //printf("%d %d %d\n", i, j, k);
         test_multiple3(&requests[i], &requests[j], &requests[k]);
       }
     }
@@ -855,9 +1161,9 @@ main (void)
            );
 
   printf("request scan 2/3      ");
-  test_scan( &requests[GET_FUNKY_CONTENT_LENGTH]
+  test_scan( &requests[POST_CHUNKED_ALL_YOUR_BASE]
            , &requests[POST_IDENTITY_BODY_WORLD]
-           , &requests[POST_CHUNKED_ALL_YOUR_BASE]
+           , &requests[GET_FUNKY_CONTENT_LENGTH]
            );
 
   printf("request scan 3/3      ");
@@ -868,6 +1174,5 @@ main (void)
 
   puts("requests okay");
 
-
   return 0;
 }
