@@ -1,4 +1,4 @@
-var sys = require("/sys.js");
+var sys = require('sys');
 
 var CRLF = "\r\n";
 var STATUS_CODES = {
@@ -109,13 +109,16 @@ exports.parseUri.options = {
 };
 
 
+var connection_expression = /Connection/i;
+var transfer_encoding_expression = /Transfer-Encoding/i;
 var close_expression = /close/i;
 var chunk_expression = /chunk/i;
+var content_length_expression = /Content-Length/i;
 
 
 /* Abstract base class for ServerRequest and ClientResponse. */
 function IncomingMessage (connection) {
-  node.EventEmitter.call(this);
+  process.EventEmitter.call(this);
 
   this.connection = connection;
   this.httpVersion = null;
@@ -136,7 +139,8 @@ function IncomingMessage (connection) {
   this.statusCode = null;
   this.client = this.connection;
 }
-node.inherits(IncomingMessage, node.EventEmitter);
+sys.inherits(IncomingMessage, process.EventEmitter);
+exports.IncomingMessage = IncomingMessage;
 
 IncomingMessage.prototype._parseQueryString = function () {
   var parts = this.uri.queryString.split('&');
@@ -177,7 +181,7 @@ IncomingMessage.prototype._addHeaderLine = function (field, value) {
 };
 
 function OutgoingMessage () {
-  node.EventEmitter.call(this);
+  process.EventEmitter.call(this);
 
   this.output = [];
   this.outputEncodings = [];
@@ -191,7 +195,8 @@ function OutgoingMessage () {
 
   this.finished = false;
 }
-node.inherits(OutgoingMessage, node.EventEmitter);
+sys.inherits(OutgoingMessage, process.EventEmitter);
+exports.OutgoingMessage = OutgoingMessage;
 
 OutgoingMessage.prototype.send = function (data, encoding) {
   var length = this.output.length;
@@ -242,15 +247,15 @@ OutgoingMessage.prototype.sendHeaderLines = function (first_line, headers) {
 
     message_header += field + ": " + value + CRLF;
 
-    if ("connection" === field) {
+    if (connection_expression.exec(field)) {
       sent_connection_header = true;
       if (close_expression.exec(value)) this.closeOnFinish = true;
 
-    } else if ("transfer-encoding" === field) {
+    } else if (transfer_encoding_expression.exec(field)) {
       sent_transfer_encoding_header = true;
       if (chunk_expression.exec(value)) this.chunked_encoding = true;
 
-    } else if ("content-length" === field) {
+    } else if (content_length_expression.exec(field)) {
       sent_content_length_header = true;
 
     }
@@ -281,7 +286,7 @@ OutgoingMessage.prototype.sendHeaderLines = function (first_line, headers) {
 
 OutgoingMessage.prototype.sendBody = function (chunk, encoding) {
   if (this.chunked_encoding) {
-    this.send(chunk.length.toString(16));
+    this.send(process._byteLength(chunk, encoding).toString(16));
     this.send(CRLF);
     this.send(chunk, encoding);
     this.send(CRLF);
@@ -313,7 +318,8 @@ function ServerResponse () {
   this.should_keep_alive = true;
   this.use_chunked_encoding_by_default = true;
 }
-node.inherits(ServerResponse, OutgoingMessage);
+sys.inherits(ServerResponse, OutgoingMessage);
+exports.ServerResponse = ServerResponse;
 
 ServerResponse.prototype.sendHeader = function (statusCode, headers) {
   var reason = STATUS_CODES[statusCode] || "unknown";
@@ -333,9 +339,10 @@ function ClientRequest (method, uri, headers) {
   }
   this.closeOnFinish = true;
 
-  this.sendHeaderLines(method + " " + uri + " HTTP/1.0\r\n", headers);
+  this.sendHeaderLines(method + " " + uri + " HTTP/1.1\r\n", headers);
 }
-node.inherits(ClientRequest, OutgoingMessage);
+sys.inherits(ClientRequest, OutgoingMessage);
+exports.ClientRequest = ClientRequest;
 
 ClientRequest.prototype.finish = function (responseListener) {
   this.addListener("response", responseListener);
@@ -344,7 +351,7 @@ ClientRequest.prototype.finish = function (responseListener) {
 
 
 function createIncomingMessageStream (connection, incoming_listener) {
-  var stream = new node.EventEmitter();
+  var stream = new process.EventEmitter();
 
   stream.addListener("incoming", incoming_listener);
 
@@ -356,7 +363,7 @@ function createIncomingMessageStream (connection, incoming_listener) {
   });
 
   // Only servers will get URI events.
-  connection.addListener("uri", function (data) {
+  connection.addListener("url", function (data) {
     incoming.uri.full += data;
   });
 
@@ -434,7 +441,7 @@ function flushMessageQueue (connection, queue) {
 
     while (message.output.length > 0) {
       if (connection.readyState !== "open" && connection.readyState !== "writeOnly") {
-        return false;
+        return true;
       }
 
       var data = message.output.shift();
@@ -455,7 +462,7 @@ function flushMessageQueue (connection, queue) {
 
 
 exports.createServer = function (requestListener, options) {
-  var server = new node.http.Server();
+  var server = new process.http.Server();
   //server.setOptions(options);
   server.addListener("request", requestListener);
   server.addListener("connection", connectionListener);
@@ -467,6 +474,8 @@ function connectionListener (connection) {
   // we need to keep track of the order they were sent.
   var responses = [];
 
+  connection.resetParser();
+
   // is this really needed?
   connection.addListener("eof", function () {
     if (responses.length == 0) {
@@ -476,6 +485,7 @@ function connectionListener (connection) {
     }
   });
 
+
   createIncomingMessageStream(connection, function (incoming, should_keep_alive) {
     var req = incoming;
 
@@ -494,7 +504,7 @@ function connectionListener (connection) {
 
 
 exports.createClient = function (port, host) {
-  var client = new node.http.Client();
+  var client = new process.http.Client();
 
   var requests = [];
 
@@ -506,14 +516,13 @@ exports.createClient = function (port, host) {
         return;
       }
       //sys.debug("client flush  readyState = " + client.readyState);
-      if (req == requests[0]) {
-        if(flushMessageQueue(client, [req])) client.close();
-      }
+      if (req == requests[0]) flushMessageQueue(client, [req]);
     });
     requests.push(req);
   };
 
   client.addListener("connect", function () {
+    client.resetParser();
     requests[0].flush();
   });
 
@@ -552,31 +561,31 @@ exports.createClient = function (port, host) {
   return client;
 };
 
-node.http.Client.prototype.get = function (uri, headers) {
+process.http.Client.prototype.get = function (uri, headers) {
   var req = new ClientRequest("GET", uri, headers);
   this._pushRequest(req);
   return req;
 };
 
-node.http.Client.prototype.head = function (uri, headers) {
+process.http.Client.prototype.head = function (uri, headers) {
   var req = new ClientRequest("HEAD", uri, headers);
   this._pushRequest(req);
   return req;
 };
 
-node.http.Client.prototype.post = function (uri, headers) {
+process.http.Client.prototype.post = function (uri, headers) {
   var req = new ClientRequest("POST", uri, headers);
   this._pushRequest(req);
   return req;
 };
 
-node.http.Client.prototype.del = function (uri, headers) {
+process.http.Client.prototype.del = function (uri, headers) {
   var req = new ClientRequest("DELETE", uri, headers);
   this._pushRequest(req);
   return req;
 };
 
-node.http.Client.prototype.put = function (uri, headers) {
+process.http.Client.prototype.put = function (uri, headers) {
   var req = new ClientRequest("PUT", uri, headers);
   this._pushRequest(req);
   return req;
@@ -584,7 +593,7 @@ node.http.Client.prototype.put = function (uri, headers) {
 
 
 exports.cat = function (url, encoding, headers) {
-  var promise = new node.Promise();
+  var promise = new process.Promise();
 
   encoding = encoding || "utf8";
 
