@@ -88,7 +88,7 @@ static Local<Object> BuildStatsObject(struct stat * s) {
   stats->Set(RDEV_SYMBOL, Integer::New(s->st_rdev));
 
   /* total size, in bytes */
-  stats->Set(SIZE_SYMBOL, Integer::NewFromUnsigned(s->st_size));
+  stats->Set(SIZE_SYMBOL, Integer::New(s->st_size));
 
   /* blocksize for filesystem I/O */
   stats->Set(BLKSIZE_SYMBOL, Integer::New(s->st_blksize));
@@ -118,6 +118,10 @@ int EIOPromise::After(eio_req *req) {
     Local<Value> exception = Exception::Error(
         String::NewSymbol(strerror(req->errorno)));
     promise->EmitError(1, &exception);
+    if (req->type == EIO_WRITE) {
+      assert(req->ptr2);
+      delete [] req->ptr2;
+    }
     return 0;
   }
 
@@ -230,9 +234,6 @@ static Handle<Value> Rename(const Arguments& args) {
   String::Utf8Value new_path(args[1]->ToString());
 
   return scope.Close(EIOPromise::Rename(*path, *new_path));
-  Promise *promise = EIOPromise::Create();
-
-  return scope.Close(promise->Handle());
 }
 
 static Handle<Value> Unlink(const Arguments& args) {
