@@ -1,5 +1,5 @@
-node.mixin(require("common.js"));
-http = require("/http.js");
+process.mixin(require("./common"));
+http = require("http");
 PORT = 8888;
 
 var body = "exports.A = function() { return 'A';}";
