@@ -1,5 +1,5 @@
-node.mixin(require("common.js"));
-http = require("/http.js");
+process.mixin(require("./common"));
+http = require("http");
 var PORT = 18032;
 
 var sent_body = "";
