@@ -1,6 +1,6 @@
-node.mixin(require("common.js"));
-tcp = require("/tcp.js");
-http = require("/http.js");
+process.mixin(require("./common"));
+tcp = require("tcp");
+http = require("http");
 
 var port = 8222;
 
@@ -13,12 +13,6 @@ http.createServer(function (req, res) {
   res.id = request_number;
   req.id = request_number++;
 
-  puts("server got request " + req.id);
-
-  req.addListener("complete", function () {
-    puts("request complete " + req.id);
-  });
-
   if (req.id == 0) {
     assertEquals("GET", req.method);
     assertEquals("/hello", req.uri.path);
@@ -30,11 +24,10 @@ http.createServer(function (req, res) {
     assertEquals("POST", req.method);
     assertEquals("/quit", req.uri.path);
     this.close();
-    puts("server closed");
+    //puts("server closed");
   }
 
   setTimeout(function () {
-    puts("send response " + req.id);
     res.sendHeader(200, {"Content-Type": "text/plain"});
     res.sendBody(req.uri.path);
     res.finish();
@@ -47,8 +40,7 @@ var c = tcp.createConnection(port);
 c.setEncoding("utf8");
 
 c.addListener("connect", function () {
-  puts("client connected. sending first request");
-  c.send("GET /hello?hello=world&foo=b==ar HTTP/1.1\r\n\r\n" );
+  c.send( "GET /hello?hello=world&foo=b==ar HTTP/1.1\r\n\r\n" );
   requests_sent += 1;
 });
 
@@ -56,9 +48,7 @@ c.addListener("receive", function (chunk) {
   server_response += chunk;
 
   if (requests_sent == 1) {
-    puts("send request 2");
     c.send("POST /quit HTTP/1.1\r\n\r\n");
-    puts("close client");
     c.close();
     assertEquals(c.readyState, "readOnly");
     requests_sent += 1;
@@ -66,12 +56,10 @@ c.addListener("receive", function (chunk) {
 });
 
 c.addListener("eof", function () {
-  puts("client got eof");
   client_got_eof = true;
 });
 
 c.addListener("close", function () {
-  puts("client closed");
   assertEquals(c.readyState, "closed");
 });
 
