@@ -1,8 +1,8 @@
-node.mixin(require("common.js"));
-http = require("/http.js");
+process.mixin(require("./common"));
+http = require("http");
 
-var multipart = require('/multipart.js');
-var fixture = require('fixtures/multipart.js');
+var multipart = require('multipart');
+var fixture = require('./fixtures/multipart');
 
 var port = 8222;
 var parts_reveived = 0;
@@ -15,7 +15,7 @@ var server = http.createServer(function(req, res) {
   stream.addListener('part', function(part) {
     parts_reveived++;
 
-    var name = part.headers['content-disposition'].name;
+    var name = part.name;
 
     if (parts_reveived == 1) {
       assertEquals('reply', name);
@@ -28,6 +28,7 @@ var server = http.createServer(function(req, res) {
       parts[name] += chunk;
     });
     part.addListener('complete', function(chunk) {
+      assertEquals(0, part.buffer.length);
       if (parts_reveived == 1) {
         assertEquals('yes', parts[name]);
       } else if (parts_reveived == 2) {
