@@ -1,13 +1,13 @@
-node.mixin(require("common.js"));
+process.mixin(require("./common"));
 
 var got_error = false;
 
-var promise = node.fs.readdir(fixturesDir);
+var promise = posix.readdir(fixturesDir);
 puts("readdir " + fixturesDir);
 
 promise.addCallback(function (files) {
   p(files);
-  assertArrayEquals(["a.js", "b", "multipart.js", "x.txt"], files.sort());
+  assertArrayEquals(["a.js", "b", "multipart.js", "test_ca.pem", "test_cert.pem", "test_key.pem", "x.txt"], files.sort());
 });
 
 promise.addErrback(function () {
