@@ -1,4 +1,4 @@
-node.mixin(require("common.js"));
+process.mixin(require("./common"));
 
 var WINDOW = 200; // why is does this need to be so big?
 
